package com.sd.business.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.util.ListUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.dto.PurchaseSelectDto;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.PurchaseBomVo;
import com.sd.business.entity.purchase.vo.PurchaseVo;
import com.sd.business.mapper.purchase.PurchaseMapper;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购合同 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase> implements PurchaseService {
    @Autowired
    private PurchaseBomService purchaseBomService;

    @Override
    public Page<PurchaseVo> getPage(PurchaseSelectDto dto) {
        IWrapper<Purchase> wrapper = getWrapper();
        //采购单号
        wrapper.like("p",Purchase::getCode,dto.getCode());
        //采购单价
        wrapper.eq("p",Purchase::getFlowStatus,dto.getFlowStatus());
        //供应商名称
        wrapper.like("s.name",dto.getSupplierName());
        wrapper.orderByDesc("p", Purchase::getCreateTime);
        //查询采购单数据
        Page<PurchaseVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<PurchaseVo> records = page.getRecords();

        //获取到采购单ID
        List<Long> ids = records.stream().map(PurchaseVo::getId).collect(Collectors.toList());
        //查询采购单详情数据
        LambdaQueryWrapper<PurchaseBom> query = Wrappers.lambdaQuery();
        query.in(PurchaseBom::getPurchaseId,ids);
        List<PurchaseBomVo> purchaseBomVoList = purchaseBomService.getList(query);
        Map<Long, List<PurchaseBomVo>> purchaseBomVoMap = purchaseBomVoList.stream().collect(Collectors.groupingBy(PurchaseBomVo::getPurchaseId));

        //赋值采购详情数据
        for (PurchaseVo record : records) {
            List<PurchaseBomVo> purchaseBomVos = purchaseBomVoMap.get(record.getId());
            if (ObjectUtil.isEmpty(purchaseBomVos)){
                record.setPurchaseBomVoList(purchaseBomVos);
            }
        }
        return page;
    }

    @Override
    public PurchaseVo detail(Long id) {
        Purchase Purchase = this.getById(id);
        PurchaseVo result = BeanUtil.toBean(Purchase, PurchaseVo.class);
        return result;
    }

    @Override
    public void add(PurchaseDto purchaseDto) {
        this.save(purchaseDto);
    }

    @Override
    public void edit(PurchaseDto purchaseDto) {
        this.updateById(purchaseDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
